'''
Ce programme vise à montrer comment utiliser la classe ComposedClassifier afin de classifier des signaux EEG
a l'aide d'un classifieur d'ensemble basé sur le vote à la majorité entre les réseaux EEGNet, DeepConvNet et
ShallowConvNet de EEGModels.
'''
import os

import numpy as np
import threading

class PredictionThread(threading.Thread):
    def __init__(self,i,model,epochs):
        threading.Thread.__init__(self)
        self.result = None
        self.model = model
        self.epochs = epochs
        self.id = i

    def run(self):
        print("Starting thread %d..."%self.id)
        self.result = self.model.predict(self.epochs)
        print("End of thread %d..." % self.id)

class ComposedClassifier:

    def __init__(self,models_lst,classifier_id):
        self.models = models_lst
        self.id = classifier_id
        print("Composed classifier (id : %s) successfully initialized."%self.id)
        print("Number of models that are part of the composed classifier : ",len(self.models))
        print("Model types : ")
        for model in self.models:
            print("- " + model.model_type)

    def train(self,epochs,labels,load_models,epochs_nb):
        print("Training a the composed classifier...")
        if not os.path.exists(self.id+"_model_data"):
            os.makedirs(self.id+"_model_data")

        for i in range(len(self.models)):
            if self.models[i].model_type == "FBCSP":
                filename = os.path.join(self.id+"_model_data",self.id)
                self.models[i].train(epochs,labels,filename)
            else:
                filename = os.path.join(self.id+"_model_data",self.id + "_" + self.models[i].model_type + "_composed.h5")
                self.models[i].train(load_model=load_models,epochs=epochs,labels=labels,epochs_number=epochs_nb,save_as=filename)

    def predict(self,epochs):
        predictions = []
        threads = []

        for i in range(len(self.models)):
            t1 = PredictionThread(i,self.models[i],epochs)
            threads.append(t1)
            t1.start()

        for i in range(len(threads)):
            threads[i].join()
            predictions.append(threads[i].result)


        final_pred = []
        for i in range(len(predictions[0])):
            current_preds = []
            for k in range(len(self.models)):
                current_preds.append(predictions[k][i])
            unique, counts = np.unique(current_preds, return_counts=True)
            final_pred.append(unique[counts.argmax()])

        return final_pred

    def load_model(self):
        print("Loading models for a composed classifier...")
        filenames = []
        for i in range(len(self.models)):
            if self.models[i].model_type == "FBCSP":
                filename = os.path.join(self.id+"_model_data",self.id)
            else:
                filename = os.path.join(self.id+"_model_data",self.id + "_" + self.models[i].model_type + "_composed.h5")
            if self.models[i].model_type == "FBCSP":
                filenames.append(filename + ".svrlst")
                filenames.append(filename + ".fbcsp")
            else:
                filenames.append(filename)
            self.models[i].load_model(filename)

        print("Composed classifier successfully loaded from files : ")
        for filename in filenames:
            print("- " + filename)







