import os
from math import floor

import numpy as np
import pandas as pd

from API import API, testFIF
from crypt_utils import FileEncrypter, FileDecrypter
from os.path import exists as fileExist
from mne.io import read_raw_fif as readFif
from mne import Epochs


def readFromFIF(filename):
    if (not fileExist(API.foldername + "/" + filename + API.extension)):
        return None

    raw = readFif(filename)
    data = raw.get_data()
    L = len(data[0])
    freq = API.Freq_sampling
    epo = Epochs(raw, [[0, 1, 0]], tmin=0, tmax=(L - 1) / freq, baseline=(0, (L - 1) / 125), on_missing="warn",
                 reject_by_annotation=False, preload=True, verbose="WARNING")

    nb_epo = floor(L / freq)
    epo._data = []

    for i in range(nb_epo):
        epo._data.append([])
        for j in range(16):
            epo._data[i].append(data[j][i * freq:(i + 1) * freq])
    epo._data = np.array(epo._data)
    return epo

class PersonalDataManager:
    def __init__(self):
        self.encrypter = FileEncrypter()
        self.decrypter = FileDecrypter()

    def remove_data(self,name):
        hashname = self.encrypter.name_encrypt(name)
        nb_removed = 0
        if self.decrypter.find_decrypted_file(name,"fif") is not None:
            os.remove(hashname + "_decrypted.fif")
            print(hashname + "_decrypted.fif" + " successfully removed.\n ")
            nb_removed += 1
        if self.encrypter.find(name) is not None:
            os.remove(hashname + ".crypt")
            print(hashname + ".crypt" + " successfully removed.\n ")
            nb_removed += 1
        if self.encrypter.find_key(name) is not None:
            os.remove(hashname + ".key")
            print(hashname + ".key" + " successfully removed.\n ")
            nb_removed += 1
        if nb_removed > 0 :
            print("Personal EEG data removing process ended.")
            print(nb_removed + " files have been removed.")
        else:
            print("No personal EEG data found. Please check again the input name and its format.")
            print("The correct name format is : firstname_lastname. The input name has to be lower case.")
            return False
        return True

    def generate_csv_from_matrix(self,name,matrix):
        pd.DataFrame(matrix).to_csv(name + '.csv')
        return True

    def access_data(self,name):
        api = API()
        hashname = self.encrypter.name_encrypt(name)
        if self.decrypter.find_decrypted_file(name,"fif") is not None:
            content = API.readFromFIF(hashname).get_data()
            self.generate_csv_from_np(name + ".csv",content)
        else:
            if self.encrypter.find(name) is None:
                print("No personal EEG data found. Please check again the input name and its format.")
                print("The correct name format is : firstname_lastname. The input name has to be lower case.")
                return False
            if self.encrypter.find(name) is None:
                print("No key file found for this name. Please check again the input name and its format. ")
                print("The correct name format is : firstname_lastname. The input name has to be lower case.")
                print("Be also sure that the key file is in the same directory as this program.")
                return False
            self.decrypter.decrypt_dataset(hashname)
            if self.decrypter.find_decrypted_file(name,"fif") is None:
                print("Error during file decryption.")
                return False
            else:
                content = api.readFromFIF(hashname + "_decrypted.fif").get_data()
                self.generate_csv_from_np(name + ".csv", content)
                print("CSV file successfully generated !")
                print("Path : " + name + ".csv")
                os.remove(hashname + "_decrypted.fif")
        return True


dataManager = PersonalDataManager()
dataManager.access_data("Dylan_raw.fif")
