'''
Ceci est l'application de démonstration qui sera présentée lors de la Journée Portes Ouvertes de l'ENSIM.
Elle sélectionne des signaux du dataset BCI Competition 2008 2a et les classifie.

Les résultats sont affichés de manière graphique dans une fenêtre pygame afin que les visiteurs puissent
apprécier les performances de l'intelligence artificielle utilisée et comprendre comment notre programme
classifie les signaux en temps réel.

Enfin, le programme, après classification, appuie sur une touche afin de générer un mouvement du curseur
gauche lorsque l'application Neuropong développée par l'équipe projet est lancée en paralèle.

Les visiteurs pourront alors comprendre non seulement comment nous pouvons classifier les signaux mais aussi voir
concrètement à quoi ressemblera notre réalisation finale.
'''

from EEGModels import EEGNet

import tensorflow
import time

from matplotlib import pyplot as plt

import numpy as np

from mne_utils import get_epochs_and_labels

import keyboard

import pygame

epochs,labels = get_epochs_and_labels(3,"T")

current_time = 1

pygame.init()
pygame.font.init()
font = pygame.font.SysFont('Comic Sans MS', 30)

ecran = pygame.display.set_mode((600, 475))

# Ce programme tourne en arrière-plan ("démon"), il boucle donc infiniment
while True:

    # Boucle pour sélectionner l'ensemble des essais réalisés par le sujet considéré
    for i in range(1,287):

        # Extraction des données issues du dataset contenues dans l'objet epochs
        X = epochs.get_data() * 1000  # format is in (trials, channels, samples)
        y = labels

        kernels, chans, samples = 1, 22, 1901

        X_test = X[:, ]
        Y_test = y[:]

        Y_test = tensorflow.keras.utils.to_categorical(Y_test - 1)

        # Conversion au format Essais, electrodes, données, noyaux
        X_test = X_test.reshape(X_test.shape[0], chans, samples, kernels)

        # Configuration du modèle
        model = EEGNet(nb_classes=4, Chans=chans, Samples=samples,
                           dropoutRate=0.5, kernLength=32, F1=8, D=2, F2=16,
                           dropoutType='Dropout')

        # Définition de la fonction de loss et de l'optimiseur utilisé
        model.compile(loss='categorical_crossentropy', optimizer='adam',
                      metrics=['accuracy'])

        # Nombre de paramètres utilisés par le modèle
        numParams = model.count_params()

        # Sauvegarde du meilleur modèle courant dans un fichier checkpoint.h5
        checkpointer = tensorflow.keras.callbacks.ModelCheckpoint(filepath='../checkpoint.h5', verbose=1,
                                                                  save_best_only=True)

        # Poids associés à chaque classe (1 par défaut car le dataset est équilibré entre les classes)
        class_weights = {0: 1, 1: 1, 2: 1, 3: 1, 4:1}

        # Chargement du modèle pré-entraîné
        model.load_weights('model_eegnet.h5')

        # Génération de l'image du signal en train d'être classifié via matplotlib.pyplot
        plt.plot(np.arange(0*current_time,1*current_time,1/250),epochs.get_data()[i,8,3*250:4*250])
        plt.axis('off')
        plt.savefig("current_signal.jpg")
        plt.clf()

        # Affichage du signal en train d'être traité
        image = pygame.image.load("../current_signal.jpg").convert()
        ecran.blit(image,(-20,0))

        # Prédiction sur les données de test issues du dataset pour le sujet courant
        expected = labels[i] - 1
        probs = model.predict(X_test[i].reshape(1, chans, samples, kernels))
        preds = probs.argmax(axis=-1)

        # Affichage dans la fenêtre pygame du mouvement imaginé qui a été réalisé dans la pratique
        if(expected[0] == 0):
            print("\n\nMouvement imaginé réalisé : Main gauche")
            text = font.render('Mouvement imaginé réalisé : Main gauche', False, (0, 0, 0))
            ecran.blit(text, (20, 0))
        elif(expected[0] == 1):
            print("\n\nMouvement imaginé réalisé : Main droite")
            text = font.render('Mouvement imaginé réalisé : Main droite', False, (0, 0, 0))
            ecran.blit(text, (20, 0))
        else:
            print("\n\nMouvement imaginé réalisé : Aucun")
            text = font.render('Mouvement imaginé réalisé : Aucun', False, (0, 0, 0))
            ecran.blit(text, (20, 0))

        # Affichage dans la fenêtre pygame du mouvement imaginé prédit par l'IA
        if(preds[0] == 0):
            print("\nMouvement imaginé prédit : Main gauche")
            text = font.render('Mouvement imaginé prédit : Main gauche', False, (0, 0, 0))
            ecran.blit(text, (20, 425))
        elif (preds[0] == 1):
            print("\nMouvement imaginé prédit : Main droite")
            text = font.render('Mouvement imaginé prédit : Main droite', False, (0, 0, 0))
            ecran.blit(text, (20, 425))
        else:
            print("\nMouvement imaginé prédit : Aucun")
            text = font.render('Mouvement imaginé prédit : Aucun', False, (0, 0, 0))
            ecran.blit(text, (20, 425))

        # Appui sur le bouton correspondant au mouvement prédit (ou aucun mouvement si 'Aucun')
        if(preds[0] == 0) :
            keyboard.press("Z")
        elif(preds[0] == 1):
            keyboard.press("S")

        # Temporisation afin de simuler un appui prolongé sur la touche et déplacer le curseur
        time.sleep(0.7)

        # Relâchement de la touche presséezszssszzzzzszsszszsszssz
        if(preds[0] == 0) :
            keyboard.release("Z")
        elif(preds[0] == 1):
            keyboard.release("S")

        # Mise à jour de la fenêtre pygame avec les nouvelles informations affichéessszsz
        pygame.display.flip()


pygame.quit()
