# -*- coding: utf-8 -*-
"""
Created on Thu Jan 13 10:33:34 2022

@author: dylan
"""

from scipy import signal
import numpy as np

# Application d'un filtre Laplacien sur les électrodes C3 et C4 (cortex moteurs gauche et droit).
def laplacian_spatial_filtering(global_data):
    # Filtre Laplacien appliqué à l'électrode C3
    global_data[7,:,1] = global_data[7,:,1] - (global_data[6,:,1] + global_data[8,:,1] + global_data[1,:,1] +
                                               global_data[13,:,1])/4

    # Filtre Laplacien appliqué à l'électrode C4
    global_data[11, :, 1] = global_data[11, :, 1] - (global_data[12, :, 1] + global_data[10, :, 1] + global_data[5, :, 1] +
                                                   global_data[17, :, 1]) / 4
    return global_data

# Application d'un filtre bipolaire sur les électrodes C3 et C4 (cortex moteurs gauche et droit).
def bipolar_spatial_filtering(global_data):
    # Filtre bipolaire appliqué à l'électrode C3
    global_data[7,:,1] = global_data[1,:,1] - global_data[13,:,1]

    # Filtre bipolaire appliqué à l'électrode C4
    global_data[11, :, 1] = global_data[5,:,1] - global_data[17,:,1]
    return global_data

# Cette fonction permet de filtrer le signal d'entrée data et d'en extraire un nombre de caractéristiques nb_feat.
# Les bornes inférieures et supérieures de filtrage doivent être spécifiées (par exemple 8-12 Hz pour la bande mu
# utilisée dans le cadre de la détection de mouvements imaginés des mains ou des pieds).
def extract_features(data,nb_feat,low_freq,high_freq):

    # Définition des paramètres pour le filtrage fréquentiel
    periode = 1/250 
    fqech = 1/periode
    f_nyq = fqech/2   

    # Filtrage fréquentiel du signal
    fc1 = low_freq
    fc2 = high_freq
    b,a = signal.butter(4,(fc1/f_nyq,fc2/f_nyq),'bandpass',analog=False)
    s_but_bp2 = signal.filtfilt(b,a,data)

    # Estimation de la puissance du signal par mise au carré
    puissance_sig = np.square(s_but_bp2)

    # Moyennage temporel du signal
    partition_size = nb_feat
    partition = int(puissance_sig.shape[0]/partition_size)
    means = []
    # Moyennage temporel par paquet de nb_feat échantillons
    for i in range(0,partition_size):
        mean = np.mean(puissance_sig[i*partition:(i+1)*partition])
        means.append(mean)
    means = np.array(means).flatten()
    
    return means