import os

from cryptography.fernet import Fernet

from hashlib import blake2b

from abc import ABC


class NameEncrypter(ABC):
    def name_encrypt(self, name):
        h = blake2b()

        h.update(name.encode())
        encrypted_filename = h.hexdigest()

        return encrypted_filename


class FileEncrypter(NameEncrypter):

    def dataset_filename_encrypt(self, name, remove_original):
        encrypted_filename = self.name_encrypt(name)

        with open(name, 'rb') as original_dataset:
            content = original_dataset.read()

        with open(encrypted_filename, 'wb') as name_encrypted_dataset:
            name_encrypted_dataset.write(content)

        if remove_original:
            os.remove(name)

    def find(self, name):
        encrypted = self.name_encrypt(name) + ".crypt"
        for file in os.listdir():
            if file == encrypted:
                return file
        return None

    def find_key(self, name):
        encrypted = self.name_encrypt(name) + ".key"
        for file in os.listdir():
            if file == encrypted:
                return file
        return None

    def encrypt_dataset(self, filename):
        with open(filename, "rb") as file:
            lines = file.read()

            key = Fernet.generate_key()

            fernet = Fernet(key)

            enctex = fernet.encrypt(lines)

            with open(filename + '.crypt', 'wb') as encrypt_file:
                encrypt_file.write(enctex)

            with open(filename + '.key', 'wb') as key_file:
                key_file.write(key)

        os.remove(filename)


class FileDecrypter(NameEncrypter):

    def decrypt_dataset(self, filename, filetype):
        with open(filename + '.crypt', 'rb') as encrypted_file:
            with open(filename + '.key', 'rb') as key_file:
                retrieved_key = key_file.read()
            to_decrypt = encrypted_file.read()
            retrieved_fernet = Fernet(retrieved_key)
            dectex = retrieved_fernet.decrypt(to_decrypt)

        with open(filename + '_decrypted.' + filetype, 'wb') as decrypt_file:
            decrypt_file.write(dectex)

    def find_decrypted_file(self, name, filetype):
        encrypted = self.name_encrypt(name) + "_decrypted." + filetype
        for file in os.listdir():
            if file == encrypted:
                return file
        return None
