"""
Programme permettant d'entraîner un réseau de neurones EEGNet afin de classifier les signaux issus du dataset
BCI Competition 2008 2a.
Lors de l'entraînement, le programme génère un fichier checkpoint.h5 contenant le meilleur modèle obtenu lors de
celui-ci : il peut être sauvegardé après l'entraînement afin de l'utiliser plus tard.
"""
import tensorflow
from mne.utils.docs import tf

from EEGModels import EEGNet, DeepConvNet, ShallowConvNet
from tensorflow.keras import utils as np_utils
from tensorflow.keras.callbacks import ModelCheckpoint

import scipy.io
from os.path import join as pjoin
import mne
import numpy as np

from mne_utils import get_epochs_and_labels

num_suj = 3
epochs,labels = get_epochs_and_labels(num_suj,"E")

# Extraction des données de la variable epochs
X = epochs.get_data() * 1000  # format is in (trials, channels, samples)
y = labels

kernels, chans, samples = 1, X.shape[1], X.shape[2]

# Découpage des données sous la forme de jeux d'entraînement, de validation et de test
part1 = 260
part2 = 280

X_train = X[0:part1,]
Y_train = y[0:part1]
X_validate = X[part1:part2, ]
Y_validate = y[part1:part2]
X_test = X[part1:, ]
Y_test = y[part1:]

Y_train = np_utils.to_categorical(Y_train - 1)
Y_validate = np_utils.to_categorical(Y_validate - 1)
Y_test = np_utils.to_categorical(Y_test - 1)

# Conversion au format Essais, electrodes, données, noyaux

X_train = X_train.reshape(X_train.shape[0], chans, samples, kernels)
X_validate = X_validate.reshape(X_validate.shape[0], chans, samples, kernels)
X_test = X_test.reshape(X_test.shape[0], chans, samples, kernels)

model_choice = ""
while model_choice != "EEGNet" and model_choice != "DeepConvNet" and model_choice != "ShallowConvNet":
    model_choice = input("\n\nChoose the model that you want to train (EEGNet, DeepConvNet ou ShallowConvNet) : ")


# Configuration du modèle selon celui choisi par l'utilisateur
if(model_choice == "EEGNet"):
    model = EEGNet(nb_classes=4, Chans=chans, Samples=samples,
                   dropoutRate=0.5, kernLength=32, F1=8, D=2, F2=16,
                   dropoutType='Dropout')
elif model_choice == "DeepConvNet":
    model = DeepConvNet(nb_classes=4, Chans=chans, Samples=samples)
else:
    model = ShallowConvNet(nb_classes=4, Chans=chans, Samples=samples)

# Définition de la fonction de loss et de l'optimiseur utilisé (par défaut adam)
model.compile(loss='categorical_crossentropy', optimizer='adam',
              metrics=['accuracy',tensorflow.keras.metrics.Precision(),tensorflow.keras.metrics.Recall()])

# Nombre de paramètres utilisés par le modèle
numParams = model.count_params()

# Sauvegarde du meilleur modèle courant dans un fichier checkpoint.h5
checkpointer = ModelCheckpoint(filepath='checkpoint.h5', verbose=1,
                               save_best_only=True)
load_model = ""
while load_model != "Yes" and load_model != "No":
    load_model = input("\n\nDo you want to use the last trained model corresponding to this neural network ? (Yes/No) ")

# Chargement d'un modèle préalablement sauvegardé dans un fichier h5 si demandé par l'utilisateur
if(load_model == "Yes"):
    if (model_choice == "EEGNet"):
        model.load_weights('model_eegnet.h5')
    elif model_choice == "DeepConvNet":
        model.load_weights('model_deepconvnet.h5')
    else:
        model.load_weights('model_shallowconvnet.h5')

    # Calcul de l'accuracy du modèle chargé sur les données de test courantes
    probs = model.predict(X_test)
    preds = probs.argmax(axis=-1)
    acc = np.mean(preds == Y_test.argmax(axis=-1))
    print("\nClassification accuracy before training : %f " % (acc))

class_weights = {0:1,1:1,2:1,3:1}

# Entraînement du réseau de neurones
# Définition ici notamment de la taille de lot (batch_size) et du nombre d'epochs (epochs)
fittedModel = model.fit(X_train, Y_train, batch_size=8, epochs=300,
                        verbose=2, validation_data=(X_validate, Y_validate),
                        callbacks=[checkpointer], class_weight=class_weights)

# Chargement du meilleur modèle obtenu lors de l'entraînement
model.load_weights('checkpoint.h5')

# Classfication des données de test
probs = model.predict(X_test)
preds = probs.argmax(axis=-1)
acc = np.mean(preds == Y_test.argmax(axis=-1))
print("Classification accuracy: %f " % (acc))
