'''
Ce programme vise à montrer comment utiliser la classe EEGDecodingModel pour classifier des signaux EEG.
'''
import numpy as np
from sklearn import metrics

from EEGDecodingModel import EEGDecodingModel
from mne_utils import get_epochs_and_labels

# Récupération des données du dataset BCI Competition 2008 2a
num_suj = 3
epochs, labels = get_epochs_and_labels(num_suj, "E")
test_epochs, test_labels = get_epochs_and_labels(num_suj, "T")

# Découpage des données afin de ne conserver qu'un échantillon de 2 secondes pour chaque essai (plus proche du
# temps-réel).
epochs = epochs.crop(tmin=2.5,tmax=4.5)
test_epochs = test_epochs.crop(tmin=2.5,tmax=4.5)


labels = labels.flatten()

# Définition du nombre d'époques pour l'entraînement
epochs_nb = 350

# Définition d'un modèle EEGNet (voir EEGModels.py pour plus d'informations).
eegmodel = EEGDecodingModel(model_type="EEGNet", chans=epochs.get_data().shape[1], samples=epochs.get_data().shape[2],nb_classes=4)

# Entraînement du modèle
eegmodel.train(load_model=False,epochs=epochs,labels=labels,epochs_number=epochs_nb,save_as="checkpoint.h5")

# Récupération de la prédiction réalisée sur les données d'entraînement ainsi que l'accuracy et le score kappa calculés.
preds = eegmodel.predict(epochs=test_epochs,true_labels=test_labels)

acc = metrics.accuracy_score(test_labels, preds)
kappa_score = metrics.cohen_kappa_score(test_labels, preds)

print("Prediction accuracy : ",acc)
print("Prediction kappa score : ",kappa_score)