# -*- coding: utf-8 -*-
"""
Created on Sun Oct 24 16:53:02 2021

 Programme exploitant les données du dataset BCI Competition 2008 2a afin
 de tester les performances d'algorithmes de traitement du signal et de machine learning
 pour reconnaître des mouvements imaginés de la main droite, de la main gauche
 des pieds ou de la langue.

 Cette évaluation repose sur les données de 9 sujets du dataset BCI Competition 2008. Pour chaque sujet, deux
 sessions de 288 signaux EEG ont été enregistrés et leurs labels respectifs
 ont été sauvegardés.

@author: dylan
"""

from import_data import generate_usable_data
import matplotlib.pyplot as plt
import numpy as np
import mne
import sklearn
from extracting_features import extract_features,bipolar_spatial_filtering,laplacian_spatial_filtering

persons = [1,2,3,4,5,6,7,8,9]

accuracies = []
accuracies_md = []

from sklearn.ensemble import RandomForestClassifier
classifier = RandomForestClassifier(max_depth=7, criterion='gini')
# Boucle pour opérer la classification sur tous les sujets du dataset
for i in range(1,10):
    
    # Récupération des données du dataset BCI Competition 2a.
    # Pour chaque sujet, sa session d'évaluation est exploitée afin d'entraîner
    # l'algorithme et sa session de training permettra l'évaluation des méthodes.
    
    nom_sujet = "A0" + str(i) + "E"
    output_data,labels = generate_usable_data(nom_sujet)
    
    nom_sujet2 = "A0" + str(i) + "T"
    output_data2,labels2 = generate_usable_data(nom_sujet2)
     
    # Mise en places de filtres spatiaux Laplaciens sur les signaux
    new_data_train = laplacian_spatial_filtering(output_data)
    new_data_test = laplacian_spatial_filtering(output_data2)

    X_train = new_data_train[7,:,1]
    X_test = new_data_test[7,:,1]
   
    # Filtrage fréquentiel dans la bande de fréquences beta (18Hz-26Hz) puis 
    # calcul de la puissance de bande du signal
    X_train_temp = []
    X_test_temp = []
    # 50 -> scores corrects
    nb_feat = 10
    l_freq = 8
    h_freq = 28
    for i in range(0,288):
        X_train_temp.append(extract_features(X_train[i],nb_feat,l_freq,h_freq))
        X_test_temp.append(extract_features(X_test[i],nb_feat,l_freq,h_freq))
    X_train = np.array(X_train_temp)
    X_test = np.array(X_test_temp)


    # Récupération des étiquettes pour chaque signal (de test et d'évaluation)
    
    # Les différentes classes retenues via le dataset BCI Competition 2a sont :
    #   - classe 1 : mouvement imaginé de la main gauche.
    #   - classe 2 : mouvement imaginé de la main droite.
    #   - classe 3 : mouvement imaginé des pieds.
    #   - classe 4 : mouvement imaginé de la langue.
    
    
    Y_train = labels
    Y_test = labels2
    
    # Lissage du vecteur Y_train en 1D
    
    Y_train = np.ravel(Y_train)
    
    from sklearn.preprocessing import StandardScaler
    
    # Mise à la bonne échelle des données d'entrée pour la classification
    
    sc = StandardScaler()


    # Axe d'amélioration : utilisation de PCA pour la réduction de la dimension d'entrée
    '''
    from sklearn.decomposition import PCA
    pca = PCA(n_components=15)
    pca2 = PCA(n_components=15)
    X_train = pca.fit_transform(X_train)
    X_test = pca2.fit_transform(X_test)
    '''

    X_train = sc.fit_transform(X_train)
    X_test = sc.transform(X_test)
    
    from sklearn.discriminant_analysis import LinearDiscriminantAnalysis as LDA
    
    # Mise en place du classifieur LDA (Linear Discriminant Analysis)
    
    lda = LDA(n_components=3)
    X_train = lda.fit_transform(X_train, Y_train)
    X_test = lda.transform(X_test)
    
    # Exécution de la prédiction sur les données d'évaluation
    # Ajouter un paramètre random_state = something pour ne pas avoir d'initialisation aléatoire à chaque
    # exécution (définition de la seed).
    
    classifier.fit(X_train, Y_train)
    Y_pred = classifier.predict(X_test)

    Y_pred_on_train = classifier.predict(X_train)
    
    Y_test = Y_test.reshape(1,-1)

    # Calcul du taux de bonnes prédictions pour chaque classe
    nb_1 = 0
    nb_2 = 0
    nb_3 = 0
    nb_4 = 0
    for i in range(0,Y_test[0].shape[0]):
        if (Y_test[0][i] == 1 and Y_pred[i] == 1):
            nb_1 = nb_1 + 1
        elif (Y_test[0][i] == 2 and Y_pred[i] == 2):
            nb_2 = nb_2 + 1
        elif (Y_test[0][i] == 3 and Y_pred[i] == 3):
            nb_3 = nb_3 + 1
        elif (Y_test[0][i] == 4 and Y_pred[i] == 4):
            nb_4 = nb_4 + 1

    print("Predicted classes : ",Y_pred)
    print("Real classes : ",Y_test[0])

    # Calcul puis affichage de l'accuracy sur les données nouvelles et sur les données utilisées lors de
    # l'entraînement de l'IA

    from sklearn import metrics
    score = metrics.accuracy_score(Y_test.flatten(), Y_pred.flatten())
    score_on_train_data = metrics.accuracy_score(Y_train.flatten(), Y_pred_on_train.flatten())

    print("Accuracy score on new data : ", score)
    print("Accuracy score on train data : ", score_on_train_data)

    # Affichage pour chaque sujet de l'accuracy sur les 4 classes : mouvement
    # de la main droite, mouvement imaginé de la main gauche, mouvement imaginé
    # des pieds et mouvement imaginé de la langue.
    print("Good prediction rate left hand : ", nb_1/(288/4))
    print("Good prediction rate right hand : ", nb_2/(288/4))
    print("Good prediction rate feet : ", nb_3/(288/4))
    print("Good prediction rate tongue : ", nb_4/(288/4))

    accuracies.append(score)
    accuracies_md.append(nb_2/(288/4))

# Affichage et sauvegarde d'un graphique montrant les résultats de la classification des signaux EEG
plt.gcf().clear()
fig = plt.figure(1)
ax = fig.add_subplot(111)
ax.scatter(persons,accuracies,label = 'Global accuracy',marker = '.')
ax.scatter(persons,accuracies_md,label = 'Accuracy on right hand',marker = '.')
handles, labels = ax.get_legend_handles_labels()
lgd = ax.legend(handles, labels, loc='upper center', bbox_to_anchor=(0.5,-0.1))
text = ax.text(-0.2,1.05, "Good prediction rate", transform=ax.transAxes)
ax.set_title("Classification results for each subject\n")
plt.ylim(0,1)
plt.savefig("evaluation_results.jpg",bbox_inches="tight")
plt.show()


print("\nMean accuracy : ",np.mean(accuracies))




    

      
        
